﻿using Modbus.Device;
using System;
using System.Collections.Generic;
using System.IO.Ports;
using System.Windows.Forms;
using System.Linq;

namespace ProdigyModbus {
    public partial class Form1 : Form {
        List<Panel> listPanel = new List<Panel>();
        int index;
        public Form1() {
            InitializeComponent();
        }
        private void ModbusSerialRtuMasterWriteRegisters(int a) {
            string connectn = connectioncombo.SelectedItem.ToString();
            try {
                if (connectn == "RTU") {
                    string serial = connectiontextbox.Text;
                    using (SerialPort port = new SerialPort(serial)) {
                        // configure serial port
                        port.BaudRate = 9600;
                        port.DataBits = 8;
                        port.Parity = Parity.None;
                        port.StopBits = StopBits.One;
                        port.Open();
                        IModbusSerialMaster master = ModbusSerialMaster.CreateRtu(port);
                        byte slaveId = 1;
                        switch (a) {
                            case 1:
                                write_multiple_holding_register(master, slaveId);
                                break;
                            case 2:
                                read_holding_register(master, slaveId);
                                break;
                            case 3:
                                write_coil(master, slaveId);
                                break;
                            case 4:
                                read_coil(master, slaveId);
                                break;
                            case 5:
                                write_multiple_coil(master, slaveId);
                                break;
                            case 6:
                                input_register(master, slaveId);
                                break;
                            case 7:
                                discreteinput_register(master, slaveId);
                                break;

                        }
                    }
                }

            }
            catch (Exception e) {
                string msgage = "Error: Connection Error";
                MessageBox.Show(msgage);

            }
        }
     
        ///RTU METHODS///
        private void write_multiple_holding_register(IModbusSerialMaster master, byte slaveId) {
            try {
                int j;
                ushort startAddress = Convert.ToUInt16(writeholdregstart.Value);
                int s = Convert.ToInt16(writeholdregnumberreg.Value);
                ushort[] registers = new ushort[s];
                string authors = writeholdreginp.Text;
                string[] authorsList = authors.Split(' '); // Split input by space

                for (j = 0; j < s; j++) {
                    // Directly parse as ushort (decimal values)
                    registers[j] = Convert.ToUInt16(authorsList[j]);
                }
                // Write to Modbus
                master.WriteMultipleRegisters(slaveId, startAddress, registers);
            }
            catch (Exception e) {
                MessageBox.Show($"Error Occurred: {e.Message}");
            }
        }

        private void read_holding_register(IModbusSerialMaster master, byte slaveId) {
            try {
                ushort startAddress = Convert.ToUInt16(readholdregstart.Value);
                ushort numberRegisters = Convert.ToUInt16(readholdregnumber.Value);
                ushort[] registers = master.ReadHoldingRegisters(slaveId, startAddress, numberRegisters);
                // Convert each register to hexadecimal format
                string[] hexValues = registers.Select(r => r.ToString("d")).ToArray(); // "X4" formats it as a 4-digit hexadecimal
                readcoils.Text = String.Join(" , ", hexValues);
            }
            catch (Exception e) {
                MessageBox.Show($"Error: {e.Message}");
            }
        }

        private void write_coil(IModbusSerialMaster master, byte slaveId) {
            try {
                ushort coil = Convert.ToUInt16(writesinglecoilstart.Value);
                string inpt = writesinglecoilinp.Text;
                string[] inptlist = inpt.Split(' ');
                char si = Convert.ToChar(inptlist[0]);
                if (si == '1') {
                    bool a = true;
                    master.WriteSingleCoil(slaveId, coil, a);
                }
                else if (si == '0') {
                    bool a = false;
                    master.WriteSingleCoil(slaveId, coil, a);
                }
            }
            catch(Exception e) {
                MessageBox.Show("Error: Error Occured");
            }
        }

        private void write_multiple_coil(IModbusSerialMaster master, byte slaveId) {
            try {
                int i;
                ushort coil = Convert.ToUInt16(writemulticoilsstart.Value);
                ushort numbercoils = Convert.ToUInt16(writemulticoilsnumberofcoils.Value);
                string inpt = writemulticoilstextbox.Text;
                string[] inptlist = inpt.Split(' ');
                for (i = 0; i < (numbercoils); i++) {
                    char si = Convert.ToChar(inptlist[i]);
                    if (si == '1') {
                        bool a = true;
                        master.WriteSingleCoil(slaveId, coil, a);
                    }
                    if (si == '0') {
                        bool a = false;
                        master.WriteSingleCoil(slaveId, coil, a);
                    }
                    coil = ++coil;
                }
            }
            catch(Exception e) {
                MessageBox.Show("Error: Error Occured");
            }
        }

        private void read_coil(IModbusSerialMaster master, byte slaveId) {
            try {
                ushort startAddr = Convert.ToUInt16(readcoilsstart.Value);
                ushort numbercoils = Convert.ToUInt16(readnumbercoils.Value);
                string[] coilStates = new string[numbercoils];
                bool[] coils = master.ReadCoils(slaveId, startAddr, numbercoils);
                for (int i = 0; i < numbercoils; i++) {
                    coilStates[i] = coils[i] ? "1" : "0";                                   // Convert true (or) false to "1" (or) "0" and store in the array
                }
                // Join the array and display in the textbox
                readcoilstextbox.Text = String.Join(" , ", coilStates);
            }
            catch (Exception e) {
                MessageBox.Show("Error: An error occurred while reading coils.");
            }
        }

        private void input_register(IModbusSerialMaster master, byte slaveId) {
            try {
                ushort startAddr = Convert.ToUInt16(inputregisterStart.Value);
                ushort numberRegisters = Convert.ToUInt16(readinputregnumberofcoils.Value);
                ushort[] input = master.ReadInputRegisters(slaveId, startAddr, numberRegisters);
                // Convert each register to hexadecimal format
                string[] hexValues = input.Select(r => r.ToString("X4")).ToArray();         // "X4" formats it as a 4-digit hexadecimal
                inputregistertextbox.Text = String.Join(", ", hexValues);
            }
            catch (Exception e) {
                MessageBox.Show("Error: Error Occured");
            }
        }


        private void discreteinput_register(IModbusSerialMaster master, byte slaveId) {
            try {
                ushort startAddr = Convert.ToUInt16(discreteinpStart.Value);
                ushort numberCoils = Convert.ToUInt16(discreteinptnumberregs.Value);
                bool[] coils = master.ReadInputs(slaveId, startAddr, numberCoils);
                // Convert boolean values to "1" or "0" strings
                string[] values = coils.Select(c => c ? "1" : "0").ToArray();
                discreteinptextbox.Text = String.Join(" , ", values);
            }
            catch (Exception e) {
                MessageBox.Show("Error: Error Occurred");
            }
        }

        private void coilsbutton_Click(object sender, EventArgs e) {
            listPanel.Add(panel2);
            listPanel.Add(panel3);
            listPanel.Add(panel5);
            listPanel.Add(panel4);
            listPanel.Add(panel6);
            listPanel.Add(panel7);
            listPanel.Add(panel9);
            listPanel[index].BringToFront();
            if (A1.Checked) index = 0;
            else if (A2.Checked) index = 1;
            else if (A3.Checked) index = 2;
            else if (A4.Checked) index = 3;
            else if (A5.Checked) index = 4;
            else if (A6.Checked) index = 5;
            else if (A7.Checked) index = 6;

            switch (index)
            {
                case 0:
                    panel2.BringToFront();
                    break;
                case 1:
                    panel3.BringToFront();
                    break;
                case 2:
                    panel5.BringToFront();
                    break;
                case 3:
                    panel4.BringToFront();
                    break;
                case 4:
                    panel6.BringToFront();
                    break;
                case 5:
                    panel7.BringToFront();
                    break;
                case 6:
                    panel9.BringToFront();
                    break;
                default:
                    break;
            }

        }

        private void holdwr_Click(object sender, EventArgs e) {
            string connectn = connectioncombo.SelectedItem.ToString();
            switch (connectn) {
                case "RTU":
                    ModbusSerialRtuMasterWriteRegisters(1);
                    break;
            }
        }

        private void readholdregbutto_Click(object sender, EventArgs e) {
            string connectn = connectioncombo.SelectedItem.ToString();
            switch (connectn) {
                case "RTU":
                    ModbusSerialRtuMasterWriteRegisters(2);
                    break;
            }
        }

        private void writecoil_Click(object sender, EventArgs e) {
            string connectn = connectioncombo.SelectedItem.ToString();
            switch (connectn) {
                case "RTU":
                    ModbusSerialRtuMasterWriteRegisters(3);
                    break;
            }
        }

        private void readcoilbutton_Click(object sender, EventArgs e) {
            string connectn = connectioncombo.SelectedItem.ToString();
            switch (connectn) {
                case "RTU":
                    ModbusSerialRtuMasterWriteRegisters(4);
                    break;
            }
        }

        private void writemulticoilsbutton_Click(object sender, EventArgs e) {
            string connectn = connectioncombo.SelectedItem.ToString();
            switch (connectn) {
                case "RTU":
                    ModbusSerialRtuMasterWriteRegisters(5);
                    break;
            }
        }

        private void readinputregbutton_Click(object sender, EventArgs e) {
            string connectn = connectioncombo.SelectedItem.ToString();
            switch (connectn) {
                case "RTU":
                    ModbusSerialRtuMasterWriteRegisters(6);
                    break;
            }
        }

        private void discreteinpbutton_Click(object sender, EventArgs e) {
            string connectn = connectioncombo.SelectedItem.ToString();
            switch (connectn) {
                case "RTU":
                    ModbusSerialRtuMasterWriteRegisters(7);
                    break;
            }
        }

        private void label13_Click(object sender, EventArgs e) {

        }
        private void Select_Click(object sender, EventArgs e) {

        }
        private void A6_CheckedChanged(object sender, EventArgs e) {

        }
        private void A2_CheckedChanged(object sender, EventArgs e) {

        }
        private void A7_CheckedChanged(object sender, EventArgs e) {

        }
        private void A5_CheckedChanged(object sender, EventArgs e) {

        }
        private void A3_CheckedChanged(object sender, EventArgs e) {

        }
        private void Form1_Load_1(object sender, EventArgs e) {

        }
        private void A1_CheckedChanged(object sender, EventArgs e) {

        }
        private void label9_Click(object sender, EventArgs e) {

        }
        private void connectioncombo_SelectedIndexChanged(object sender, EventArgs e) {

        }
    }
}



